@@ -1,32 +1,17 @@
-# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Michal Minar <miminar@redhat.com>
 #
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
-# 1. Redistributions of source code must retain the above copyright notice,
-#    this list of conditions and the following disclaimer.
-# 2. Redistributions in binary form must reproduce the above copyright notice,
-#    this list of conditions and the following disclaimer in the documentation
-#    and/or other materials provided with the distribution.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-#
-# The views and conclusions contained in the software and documentation are
-# those of the authors and should not be interpreted as representing official
-# policies, either expressed or implied, of the FreeBSD Project.
-#
-# Authors: Michal Minar <miminar@redhat.com>
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
 """
 Meta classes simplifying declaration of user commands.
 
@@ -54,12 +39,13 @@ RE_CALLABLE = re.compile(
         re.IGNORECASE)
 RE_ARRAY_SUFFIX = re.compile(r'^(?:[a-z_]+[a-z0-9_]*)?$', re.IGNORECASE)
 RE_OPTION = re.compile(r'^-+(?P<name>[^-+].*)$')
+RE_MODULE_PATH = re.compile(r'([a-zA-z_]\w+\.)+[a-zA-z_]\w+')
 
 FORMAT_OPTIONS = ('no_headings', 'human_friendly')
 
 LOG = get_logger(__name__)
 
-def _handle_usage(name, dcl):
+def _handle_usage(name, bases, dcl):
     """
     Take care of ``OWN_USAGE`` property. Supported values:
 
@@ -96,6 +82,12 @@ def _handle_usage(name, dcl):
                     return hlp
                 dcl['get_usage'] = _new_get_usage
         has_own_usage = True
+    elif (   dcl.get('__node__', None) is None
+         and any(getattr(b, 'has_own_usage', lambda: False)() for b in bases)):
+        # inherit doc string of base class
+        dcl['__doc__'] = (  b.__doc__ for b in bases
+                         if getattr(b, 'has_own_usage', lambda: False)()).next()
+        has_own_usage = True
     if has_own_usage:
         if not 'has_own_usage' in dcl:
             dcl['has_own_usage'] = classmethod(lambda _cls: True)
@@ -133,6 +125,19 @@ def _handle_namespace(dcl):
             return namespace
         dcl['cim_namespace'] = classmethod(_new_cim_namespace)
 
+def _handle_connection_timeout(dcl):
+    """
+    Overrides ``connection_timeout()`` class method if ``CONNECTION_TIMEOUT``
+    property is given.
+
+    :param dictionary dcl: Class dictionary being modified by this method.
+    """
+    if 'CONNECTION_TIMEOUT' in dcl:
+        timeout = dcl.pop('CONNECTION_TIMEOUT')
+        def _new_connection_timeout(_cls):
+            return timeout
+        dcl['connection_timeout'] = classmethod(_new_connection_timeout)
+
 def _handle_callable(name, bases, dcl):
     """
     Process the ``CALLABLE`` property of end-point command. Create the
@@ -436,7 +441,7 @@ def _handle_fallback_command(name, bases, dcl):
                     usage_string = cmd.__doc__
             if not usage_string:
                 errors.LmiCommandError(dcl['__module__'], name,
-                        "missing usage string")
+                        "Missing usage string.")
             fallback.__doc__ = usage_string
             fallback.has_own_usage = lambda cls: True
         dcl['fallback_command'] = staticmethod(lambda: fallback)
@@ -455,7 +460,7 @@ def _handle_format_options(name, bases, dcl):
             if opt_name not in FORMAT_OPTIONS:
                 raise errors.LmiCommandInvalidProperty(
                         dcl['__module__'], name,
-                        'formatting option "%s" is not supported' %
+                        'Formatting option "%s" is not supported.' %
                         opt_name)
             if (   opt_name in ('no_headings', 'human_friendly')
                and not isinstance(value, bool)):
@@ -480,6 +485,85 @@ def _handle_format_options(name, bases, dcl):
     for key in format_options:
         dcl.pop('FMT_' + key.upper())
 
+def _handle_select(name, dcl):
+    """
+    Process properties of :py:class:`.select.LmiSelectCommand`.
+    Currently handled properties are:
+
+        ``SELECT`` : ``list``
+            Is a list of pairs ``(condition, command)`` where ``condition`` is
+            an expression in *LMIReSpL* language. And ``command`` is either a
+            string with absolute path to command that shall be loaded or the
+            command class itself.
+
+            Small example: ::
+
+                SELECT = [
+                      ( 'OpenLMI-Hardware < 0.4.2'
+                      , 'lmi.scripts.hardware.pre042.PreCmd'
+                      )
+                    , ('OpenLMI-Hardware >= 0.4.2 & class LMI_Chassis == 0.3.0'
+                      , HwCmd
+                      )
+                ]
+
+            It says: Let the ``PreHwCmd`` command do the job on brokers having
+            ``openlmi-hardware`` package older than ``0.4.2``. Use the
+            ``HwCmd`` anywhere else where also the ``LMI_Chassis`` CIM class in
+            version ``0.3.0`` is available.
+
+            First matching condition wins and assigned command will be passed
+            all the arguments.
+
+        ``DEFAULT`` : ``str`` or :py:class:`~.base.LmiBaseCommand`
+            Defines fallback command used in case no condition can be
+            satisfied.
+
+    They will be turned into ``get_conditionals()`` method.
+    """
+    module_name = dcl.get('__module__', name)
+    if not 'SELECT' in dcl:
+        raise errors.LmiCommandError(module_name, name,
+                "Missing SELECT property.")
+    def inv_prop(msg, *args):
+        return errors.LmiCommandInvalidProperty(module_name, name, msg % args)
+    expressions = dcl.pop('SELECT')
+    if not isinstance(expressions, (list, tuple)):
+        raise inv_prop('SELECT must be list or tuple.')
+    if len(expressions) < 1:
+        raise inv_prop('SELECT must contain at least one condition!')
+    for index, item in enumerate(expressions):
+        if not isinstance(item, tuple):
+            raise inv_prop('Items of SELECT must be tuples, not %s!' %
+                    getattr(type(item), '__name__', 'UNKNOWN'))
+        if len(item) != 2:
+            raise inv_prop('Expected pair in SELECT on index %d!' % index)
+        expr, cmd = item
+        if not isinstance(expr, basestring):
+            raise inv_prop('Expected expression string on index %d'
+                    ' in SELECT!' % index)
+        if isinstance(cmd, basestring) and not RE_MODULE_PATH.match(cmd):
+            raise inv_prop('Second item of conditional pair on index %d'
+                    ' in SELECT does not look as an importable path!' % cmd)
+        if (   not isinstance(cmd, basestring)
+           and not issubclass(cmd, (basestring, base.LmiBaseCommand))):
+            raise inv_prop('Expected subclass of LmiBaseCommand (or its import'
+                    ' path) as a second item of a pair on index %d in SELECT!'
+                    % index)
+
+    default = dcl.pop('DEFAULT', None)
+    if isinstance(default, basestring) and not RE_MODULE_PATH.match(default):
+        raise inv_prop('DEFAULT "%s" does not look as an importable path!'
+            % default)
+    if (   default is not None and not isinstance(default, basestring)
+       and not issubclass(default, (basestring, base.LmiBaseCommand))):
+        raise inv_prop('Expected subclass of LmiBaseCommand'
+                ' (or its import path) as a value of DEFAULT!')
+    def _new_get_conditionals(self):
+        return expressions, default
+
+    dcl['get_conditionals'] = _new_get_conditionals
+
 class EndPointCommandMetaClass(abc.ABCMeta):
     """
     End point command does not have any subcommands. It's a leaf of
@@ -503,7 +587,7 @@ class EndPointCommandMetaClass(abc.ABCMeta):
     """
 
     def __new__(mcs, name, bases, dcl):
-        _handle_usage(name, dcl)
+        _handle_usage(name, bases, dcl)
         _handle_callable(name, bases, dcl)
         _handle_opt_preprocess(name, dcl)
         _handle_format_options(name, bases, dcl)
@@ -526,7 +610,7 @@ class EndPointCommandMetaClass(abc.ABCMeta):
 class SessionCommandMetaClass(EndPointCommandMetaClass):
     """
     Meta class for commands operating upon a session object.
-    All associated functions take as first argument an namespace abstraction
+    All associated functions take as first argument a namespace abstraction
     of type ``lmi.shell``.
 
     Handles following class properties:
@@ -536,10 +620,14 @@ class SessionCommandMetaClass(EndPointCommandMetaClass):
             function. Defaults to ``"root/cimv2"``. If ``False``, raw
             :py:class:`lmi.shell.LMIConnection` object will be passed to
             associated function.
+        ``CONNECTION_TIMEOUT`` : ``int``
+            Maximum number of seconds to wait for broker's response. If reached,
+            :py:class:`~lmi.shell.LMIExceptions.ConnectionError` will be raised.
     """
     def __new__(mcs, name, bases, dcl):
-        _handle_usage(name, dcl)
+        _handle_usage(name, bases, dcl)
         _handle_namespace(dcl)
+        _handle_connection_timeout(dcl)
         _handle_callable(name, bases, dcl)
 
         return EndPointCommandMetaClass.__new__(mcs, name, bases, dcl)
@@ -687,15 +775,15 @@ class MultiplexerMetaClass(abc.ABCMeta):
             # check COMMANDS property and make it a classmethod
             if not 'COMMANDS' in dcl:
                 raise errors.LmiCommandError(module_name, name,
-                        'missing COMMANDS property')
+                        'Missing COMMANDS property.')
             cmds = dcl.pop('COMMANDS')
             if not isinstance(cmds, dict):
                 raise errors.LmiCommandInvalidProperty(module_name, name,
                         'COMMANDS must be a dictionary')
             if not all(isinstance(c, basestring) for c in cmds.keys()):
                 raise errors.LmiCommandInvalidProperty(module_name, name,
-                        'keys of COMMANDS dictionary must contain command'
-                        ' names as strings')
+                        'Keys of COMMANDS dictionary must contain command'
+                        ' names as strings.')
             for cmd_name, cmd in cmds.items():
                 if not util.RE_COMMAND_NAME.match(cmd_name):
                     raise errors.LmiCommandInvalidName(
@@ -705,7 +793,7 @@ class MultiplexerMetaClass(abc.ABCMeta):
                             'COMMANDS dictionary must be composed of'
                             ' LmiBaseCommand subclasses, failed class: "%s"'
                             % cmd.__name__)
-                if not cmd.is_end_point() and not cmd.has_own_usage():
+                if cmd.is_multiplexer() and not cmd.has_own_usage():
                     LOG().warn('Command "%s.%s" is missing usage string.'
                             ' It will be inherited from parent command.',
                             cmd.__module__, cmd.__name__)
@@ -715,8 +803,39 @@ class MultiplexerMetaClass(abc.ABCMeta):
                 return cmds
             dcl['child_commands'] = classmethod(_new_child_commands)
 
-            _handle_usage(name, dcl)
+            _handle_usage(name, bases, dcl)
             _handle_fallback_command(name, bases, dcl)
             _handle_format_options(name, bases, dcl)
 
         return super(MultiplexerMetaClass, mcs).__new__(mcs, name, bases, dcl)
+
+class SelectMetaClass(abc.ABCMeta):
+    """
+    Meta class for select commands with guarded commands. Additional handled
+    properties:
+
+        ``SELECT`` : ``list``
+            List of commands guarded with expressions representing requirements
+            on server's side that need to be met.
+        ``DEFAULT`` : ``str`` or :py:class:`~.base.LmiBaseCommand`
+            Defines fallback command used in case no condition can is
+            satisfied.
+    """
+
+    def __new__(mcs, name, bases, dcl):
+        if dcl.get('__metaclass__', None) is not SelectMetaClass:
+            module_name = dcl.get('__module__', name)
+            if not '__doc__' in dcl:
+                LOG().warn('Command selector "%s.%s" is missing short'
+                    ' description string (__doc__).',
+                        module_name, name)
+                default = dcl.get('DEFAULT', None)
+                if (    default is not None
+                        and issubclass(default, base.LmiBaseCommand)
+                        and getattr(dcl['DEFAULT'], '__doc__', None)):
+                    LOG().warn('Using __doc__ string from default command for'
+                        ' selector "%s.%s".', module_name, name)
+                    dcl['__doc__'] = dcl['DEFAULT'].__doc__
+            _handle_select(name, dcl)
+        return super(SelectMetaClass, mcs).__new__(mcs, name, bases, dcl)
+
