@@ -30,7 +30,7 @@ except IOError:
 
 setup(
     name='openlmi-scripts',
-    version='0.3.0',
+    version='0.4.0',
     description='Client-side library and command-line client',
     long_description=long_description,
     author='Michal Minar',
@@ -53,11 +53,18 @@ setup(
     packages=[
         'lmi',
         'lmi.scripts',
+        'lmi.scripts._metacommand',
         'lmi.scripts.common',
         'lmi.scripts.common.command',
         'lmi.scripts.common.formatter',
-        'lmi.scripts._metacommand'],
-    install_requires=['openlmi-tools >= 0.9, < 0.9.1', 'docopt >= 0.6'],
+        'lmi.scripts.common.versioncheck'
+    ],
+    install_requires=[
+        # version of tools >= 0.10.0 contains metacommand - this package
+        # becomes redundant
+        'openlmi-tools >= 0.9, < 0.10.0',
+        'docopt >= 0.6',
+        "pyparsing"],
     include_package_data=True,
     zip_safe=False,
     entry_points={
